#-------------------------------------------
#
# Copyright (c) 2001 DataChannel, Inc. All Rights Reserved.
#
# LGPL 2.1
#
# $Id:$
#
#-------------------------------------------

from ZODBCorbaFramework import *

import Storage

import sample
import sample__POA

class SampleServer(ZODBCorbaServer, sample__POA.Server):
    """
    This is the main CORBA server.  It is created as a Persistent
    object reference.

    This Server servant is not registered in the Domain to Servants map.

    We aren't using the *args, but it could be used for auth info.
    """
    def _getSession(self, *args):
        return SampleSession(self._db)


    def _initDB(self, root):
        self._tryCommitFunction(self.__initSampleDB,
                                (root,))

    def __initSampleDB(self, root):
        # This ensures that we have the right root objects
        if not root.has_key('StorageList'):
            root['StorageList'] = Storage.StorageList()

class SampleSession(Session, sample__POA.Session):
    """
    This is the servant that implements our session.  We need to explicitly
    associate the test__POA.Session with ZODBCorbaFramework.Session in
    this subclass.
    
    This servant is also not registered in the Domain to Servants map.
    """
    def _initDB(self, db):
        Session._initDB(self, db)

    def _initDB(self, db, *args):
        self._tryCommitFunction(self.__initSampleDB, ())

    def __initSampleDB(self):
        # Store reference to root object
        self.__storageList = self._root['StorageList']

    def getStorageObjects(self):
        result = self._tryCommitFunction(self.__getStorageObjects, ())
        return self._servantLocator.getObjRefList(result)

    def __getStorageObjects(self):
        return self.__storageList.getStorageObjects()

    def getStorageObject(self, name):
        result = self._tryCommitFunction(self.__getStorageObject,
                                         (name,))
        return self._servantLocator.getObjRef(result)

    def __getStorageObject(self, name):
        return self.__storageList.getStorageObject(name)

    def createStorageObject(self):
        result = self._tryCommitFunction(self.__createStorageObject, ())
        return self._servantLocator.getObjRef(result)

    def __createStorageObject(self):
        result = Storage.StorageObject()
        self._sessionData.addObject(result)
        return result

    def nameStorageObject(self, name, sObj):
        sObj = self._servantHelper.lookupDomainObj(sObj)
        return self._tryCommitFunction(self.__nameStorageObject,
                                       (name, sObj))

    def __nameStorageObject(self, name, sObj):
        sObj.setName(name)
        self.__storageList.addStorageObject(sObj)

                
class StorageObjectServant(Servant, sample__POA.StorageObject):
    """
    This is an example of the most common case of Servant.  A pure domain
    object that is registed in the Domain to Servant Map.
    """
    def getName(self):
        return self._tryDomainCommit("getName", ())

    def read(self, offset, size):
        return self._tryDomainCommit("read",
                                     (offset, size))

    def size(self):
        return self._tryDomainCommit("size", ())
    
    def truncate(self, offset, size):
        self._tryDomainCommit("truncate",
                              (offset, size))
        
    def write(self, offset, data):
        self._tryDomainCommit("write",
                              (offset, data))

registerDomainToServantMap(Storage.StorageObject, StorageObjectServant) 
