import inspect
import linecache
import pprint
try:
    import signal
    from signal import SIGCHLD
except:
    SIGCHLD = 17
import sys
import time
from types import *

from thread import start_new_thread, get_ident

def _printLine(frame):
    filename = frame.f_code.co_filename
    lineno = frame.f_lineno
    print "    %s:%s" % (filename, lineno)
    try:
        line = linecache.getline(filename, lineno).strip()
        if line:
            print "        %s" % line
    except IOError:
        pass

def _printStack(frame):
    stack = []
    while frame != None:
        stack.append(frame)
        frame = frame.f_back
    stack.reverse()
    for frame in stack:
        _printLine(frame)

def printStack(frameOrThreadId=None):
    if not frameOrThreadId:
        frame = inspect.currentframe()
        threadId = get_ident()
    else:
        if isinstance(frameOrThreadId, FrameType):
            frame = frameOrThreadId
            threadId = 0
        else:
            threadId = frameOrThreadId
            frame = sys._getframes()[threadId]

    if threadId:
        print "Thread:%s Frame:%s" %(threadId, frame)
    else:
        print "Frame:%s" % frame

    _printStack(frame)

def printTraces():
    print
    frames = sys._getframes()
    for id, frame in frames.items():
        printStack(frame)
    print
    sys.stdout.flush()

def _handler(sigNum, frame):
    printTraces()

def _printThread():
    print "Send signal to thread %s" % get_ident()
    while 1:
        time.sleep(.5)

def installSignalHandler(sigNum=SIGCHLD):
    try:
        signal.signal(sigNum, _handler)
        start_new_thread(_printThread, ())
    except:
        assert 0, "signal not available"
    
